@@ -45,9 +45,13 @@ import java.util.Set;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Observable;
+import java.util.logging.Level;
+import java.util.logging.Logger;
+import frysk.Config;
 
 abstract public class Task
 {
+    private static Logger logger = Logger.getLogger (Config.FRYSK_LOG_ID);
     protected TaskId id;
     protected Proc proc;
 
@@ -125,16 +129,42 @@ abstract public class Task
     public boolean traceSyscall;  	// Trace syscall entry and exit
 
     /**
-     * Create a new, possibly attached, definitely running, Task.
+     * Create a new Task skeleton.
      */
-    protected Task (Proc proc, TaskId id, boolean attached)
+    private Task (TaskId id, Proc proc)
     {
 	this.proc = proc;
 	this.id = id;
-	state = TaskState.initial (this, attached);
 	proc.add (this);
 	proc.host.add (this);
     }
+    /**
+     * Create a new unattached Task.
+     */
+    protected Task (Proc proc, TaskId id)
+    {
+	this (id, proc);
+	state = TaskState.unattachedState ();
+	logger.log (Level.FINE, "{0} create unattached\n", this); 
+    }
+    /**
+     * Create a new attached clone of Task.
+     */
+    protected Task (Task task, TaskId cloneId)
+    {
+	this (cloneId, task.proc);
+	state = TaskState.clonedState (task);
+	logger.log (Level.FINE, "{0} create attached clone\n", this); 
+    }
+    /**
+     * Create a new attached main Task of Proc.
+     */
+    protected Task (Proc proc)
+    {
+	this (new TaskId (proc.getPid ()), proc);
+	state = TaskState.mainState ();
+	logger.log (Level.FINE, "{0} create attached main\n", this); 
+    }
 
     // Send operation to corresponding underlying [kernel] task.
     protected abstract void sendContinue (int sig);
@@ -163,6 +193,7 @@ abstract public class Task
      */
     void requestStop ()
     {
+	logger.log (Level.FINE, "request task stop {0}\n", Task.this); 
 	Manager.eventLoop.add (new TaskEvent ()
 	    {
 		public void execute ()
@@ -177,6 +208,7 @@ abstract public class Task
      */
     void requestContinue ()
     {
+	logger.log (Level.FINE, "request task continue {0}\n", Task.this); 
 	Manager.eventLoop.add (new TaskEvent ()
 	    {
 		public void execute ()
@@ -191,6 +223,7 @@ abstract public class Task
      */
     void requestStepInstruction ()
     {
+	logger.log (Level.FINE, "request step insn {0}\n", Task.this); 
 	Manager.eventLoop.add (new TaskEvent ()
 	    {
 		public void execute ()
@@ -214,6 +247,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task remove {0}\n", Task.this); 
 		    state = state.processPerformRemoval (Task.this);
 		}
 	    });
@@ -229,6 +263,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task attach {0}\n", Task.this); 
 		    state = state.processPerformAttach (Task.this);
 		}
 	    });
@@ -245,6 +280,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task detach {0}\n", Task.this); 
 		    state = state.processPerformDetach (Task.this);
 		}
 	    });
@@ -260,6 +296,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task stop {0}\n", Task.this); 
 		    state = state.processPerformStop (Task.this);
 		}
 	    });
@@ -275,6 +312,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task continue {0}\n", Task.this); 
 		    state = state.processPerformContinue (Task.this);
 		}
 	    });
@@ -290,22 +328,24 @@ abstract public class Task
 		Task clone = cloneArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task clone {0}\n", Task.this); 
 		    state = state.processPerformCloned (Task.this, clone);
 		}
 	    });
     }
 
     /**
-     * (internal) This task forked creating an entirely new child
-     * process.
+     * (internal) This Task forked creating an entirely new child
+     * process containing one (the fork) task.
      */
-    void performForked (final Proc forkArg)
+    void performForked (final Task forkArg)
     {
 	Manager.eventLoop.add (new TaskEvent ()
 	    {
-		Proc fork = forkArg;
+		Task fork = forkArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task fork {0}\n", Task.this); 
 		    state = state.processPerformForked (Task.this, fork);
 		}
 	    });
@@ -320,6 +360,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task stop {0}\n", Task.this); 
 		    state = state.processPerformStopped (Task.this);
 		}
 	    });
@@ -333,6 +374,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task trap {0}\n", Task.this); 
 		    state = state.processPerformTrapped (Task.this);
 		}
 	    });
@@ -347,6 +389,7 @@ abstract public class Task
 		int sig = sigArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task signalled {0}\n", Task.this); 
 		    state = state.processPerformSignaled (Task.this, sig);
 		}
 	    });
@@ -364,6 +407,7 @@ abstract public class Task
 		int value = valueArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task terminate {0}\n", Task.this); 
 		    state = state.processPerformTerminating (Task.this, signal,
 							     value);
 		}
@@ -381,6 +425,7 @@ abstract public class Task
 		Throwable w = arg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task zombied {0}\n", Task.this); 
 		    state = state.processPerformDisappeared (Task.this, w);
 		}
 	    });
@@ -395,6 +440,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task syscall {0}\n", Task.this); 
 		    state = state.processPerformSyscalled (Task.this);
 		}
 	    });
@@ -412,6 +458,7 @@ abstract public class Task
 		int value = valueArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task terminated {0}\n", Task.this); 
 		    state = state.processPerformTerminated (Task.this, signal,
 							    value);
 		}
@@ -428,6 +475,7 @@ abstract public class Task
 	    {
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "task execed {0}\n", Task.this); 
 		    state = state.processPerformExeced (Task.this);
 		}
 	    });
@@ -462,7 +510,6 @@ abstract public class Task
     {
 	return ("{" + super.toString ()
 		+ ",id=" + id
-		+ ",proc=" + proc
 		+ ",state=" + state
 		+ "}");
     }
@@ -487,6 +534,7 @@ abstract public class Task
     private void requestDeleteObserver (final TaskObservable observable,
 					final TaskObserver observer)
     {
+	logger.log (Level.FINE, "delete observer {0}\n", observer);	
 	proc.performDeleteObservation (new TaskObservation (this, observable,
 							    observer));
     }
@@ -500,6 +548,7 @@ abstract public class Task
 		Observation observation = observationArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "add observer {0}\n", observation);	
 		    state = state.processPerformAddObservation (Task.this,
 								observation);
 		}
@@ -530,6 +579,7 @@ abstract public class Task
 		TaskObserver observer = observerArg;
 		public void execute ()
 		{
+		    logger.log (Level.FINE, "unblock observer {0}\n", observer);	
 		    state = state.processRequestUnblock (Task.this, observer);
 		}
 	    });
@@ -538,12 +588,13 @@ abstract public class Task
     /**
      * Set of Cloned observers.
      */
-    private TaskObservable clonedObservers = new TaskObservable ();
+    private TaskObservable clonedObservers = new TaskObservable (this);
     /**
      * Add a TaskObserver.Cloned observer.
      */
     public void requestAddClonedObserver (TaskObserver.Cloned o)
     {
+	logger.log (Level.FINE, "clone observer {0}\n", o);	
 	requestAddObserver (clonedObservers, o);
     }
     /**
@@ -563,8 +614,10 @@ abstract public class Task
 	     i.hasNext (); ) {
 	    TaskObserver.Cloned observer
 		= (TaskObserver.Cloned) i.next ();
-	    if (observer.updateCloned (this, clone) == Action.BLOCK)
+	    if (observer.updateCloned (this, clone) == Action.BLOCK) {
 		blockers.add (observer);
+		clone.blockers.add (observer);
+	    }
 	}
 	return blockers.size ();
     }
@@ -572,7 +625,7 @@ abstract public class Task
     /**
      * Set of Attached observers.
      */
-    private TaskObservable attachedObservers = new TaskObservable ();
+    private TaskObservable attachedObservers = new TaskObservable (this);
     /**
      * Add a TaskObserver.Attached observer.
      */
@@ -606,7 +659,7 @@ abstract public class Task
     /**
      * Set of Forked observers.
      */
-    private TaskObservable forkedObservers = new TaskObservable ();
+    private TaskObservable forkedObservers = new TaskObservable (this);
     /**
      * Add a TaskObserver.Forked observer.
      */
@@ -625,14 +678,16 @@ abstract public class Task
      * Notify all Forked observers that this task forked.  Return the
      * number of blocking observers.
      */
-    int notifyForked (Proc fork)
+    int notifyForked (Task fork)
     {
 	for (Iterator i = forkedObservers.iterator ();
 	     i.hasNext (); ) {
 	    TaskObserver.Forked observer
 		= (TaskObserver.Forked) i.next ();
-	    if (observer.updateForked (this, fork) == Action.BLOCK)
+	    if (observer.updateForked (this, fork) == Action.BLOCK) {
 		blockers.add (observer);
+		fork.blockers.add (observer);
+	    }
 	}
 	return blockers.size ();
     }
@@ -640,7 +695,7 @@ abstract public class Task
     /**
      * Set of Terminated observers.
      */
-    private TaskObservable terminatedObservers = new TaskObservable ();
+    private TaskObservable terminatedObservers = new TaskObservable (this);
     /**
      * Add a TaskObserver.Terminated observer.
      */
@@ -653,6 +708,7 @@ abstract public class Task
      */
     public void requestDeleteTerminatedObserver (TaskObserver.Terminated o)
     {
+	logger.log (Level.FINE, "delete observer {0}\n", o);	
 	requestDeleteObserver (terminatedObservers, o);
     }
     /**
@@ -674,7 +730,7 @@ abstract public class Task
     /**
      * Set of Terminating observers.
      */
-    private TaskObservable terminatingObservers = new TaskObservable ();
+    private TaskObservable terminatingObservers = new TaskObservable (this);
     /**
      * Add TaskObserver.Terminating to the TaskObserver pool.
      */
@@ -708,7 +764,7 @@ abstract public class Task
     /**
      * Set of Execed observers.
      */
-    private TaskObservable execedObservers = new TaskObservable ();
+    private TaskObservable execedObservers = new TaskObservable (this);
     /**
      * Add TaskObserver.Execed to the TaskObserver pool.
      */
@@ -742,7 +798,7 @@ abstract public class Task
     /**
      * Set of Syscall observers.
      */
-    private TaskObservable syscallObservers = new TaskObservable ();
+    private TaskObservable syscallObservers = new TaskObservable (this);
     /**
      * Add TaskObserver.Syscall to the TaskObserver pool.
      */
@@ -792,7 +848,7 @@ abstract public class Task
     /**
      * Set of Signaled observers.
      */
-    private TaskObservable signaledObservers = new TaskObservable ();
+    private TaskObservable signaledObservers = new TaskObservable (this);
     /**
      * Add TaskObserver.Signaled to the TaskObserver pool.
      */
@@ -813,6 +869,7 @@ abstract public class Task
      */
     int notifySignaled (int sig)
     {
+	logger.log (Level.FINE, "notify signaled observer {0}\n", new Integer(sig));
 	for (Iterator i = signaledObservers.iterator ();
 	     i.hasNext (); ) {
 	    TaskObserver.Signaled observer
