@@ -39,6 +39,10 @@
 
 package frysk.proc;
 
+import java.util.logging.Level;
+import java.util.logging.Logger;
+import frysk.Config;
+
 /**
  * The task state machine.
  */
@@ -46,17 +50,34 @@ package frysk.proc;
 class TaskState
     extends State
 {
+    private static Logger logger = Logger.getLogger (Config.FRYSK_LOG_ID);
     /**
-     * Return the tasks initial state.
+     * Return the initial state of a detached task.
      */
-    static TaskState initial (Task task, boolean attached)
+    static TaskState unattachedState ()
     {
-	if (attached)
-	    return startRunning;
+	return unattached;
+    }
+    /**
+     * Return the initial state of the Main task.
+     */
+    static TaskState mainState ()
+    {
+	return Start.waitForStop;
+    }
+    /**
+     * Return the initial state of a cloned task.
+     */
+    static TaskState clonedState (Task parent)
+    {
+	if (parent.state == detaching)
+	    return detaching;
+	else if (parent.state == running)
+	    return Start.waitForStop;
 	else
-	    return unattached;
+	    throw new RuntimeException ("clone's parent in unexpected state "
+					+ parent);
     }
-
     protected TaskState (String state)
     {
 	super (state);
@@ -75,78 +96,97 @@ class TaskState
     }
     TaskState processPerformSignaled (Task task, int sig)
     {
+	logger.log (Level.FINE, "signal task {0}\n", task); 
 	throw unhandled (task, "PerformSignaled");
     }
     TaskState processPerformStopped (Task task)
     {
+	logger.log (Level.FINE, "stop task {0}\n", task); 
 	throw unhandled (task, "PerformStopped");
     }
     TaskState processPerformTrapped (Task task)
     {
+	logger.log (Level.FINE, "trap task {0}\n", task); 
 	throw unhandled (task, "PerformTrapped");
     }
     TaskState processPerformSyscalled (Task task)
     {
-	throw unhandled (task, "PerformSyscalled");
-    }
+	logger.log (Level.FINE, "syscall task {0}\n", task); 
+	 throw unhandled (task, "PerformSyscalled");
+     }
     TaskState processPerformTerminated (Task task, boolean signal, int value)
     {
+	logger.log (Level.FINE, "terminated task {0}\n", task); 
 	throw unhandled (task, "PerformTerminated");
     }
     TaskState processPerformTerminating (Task task, boolean signal, int value)
     {
+	logger.log (Level.FINE, "terminating task {0}\n", task); 
 	throw unhandled (task, "PerformTerminating");
     }
     TaskState processPerformExeced (Task task)
     {
+	logger.log (Level.FINE, "exec task {0}\n", task); 
 	throw unhandled (task, "PerformExeced");
     }
     TaskState processPerformDisappeared (Task task, Throwable w)
     {
-	throw unhandled (task, "PerformZombied");
+	logger.log (Level.FINE, "disappeared task {0}\n", task); 
+	throw unhandled (task, "PerformDisappeared");
     }
     TaskState processRequestStop (Task task)
     {
+	logger.log (Level.FINE, "stop task {0}\n", task); 
 	throw unhandled (task, "RequestStop");
     }
     TaskState processRequestContinue (Task task)
     {
+	logger.log (Level.FINE, "continue task {0}\n", task); 
 	throw unhandled (task, "RequestContinue");
     }
     TaskState processRequestStepInstruction (Task task)
     {
+	logger.log (Level.FINE, "step insn task {0}\n", task); 
 	throw unhandled (task, "RequestStepInstruction");
     }
     TaskState processPerformRemoval (Task task)
     {
+	logger.log (Level.FINE, "remove task {0}\n", task); 
 	throw unhandled (task, "PerformRemoval");
     }
     TaskState processPerformAttach (Task task)
     {
+	logger.log (Level.FINE, "attach task {0}\n", task); 
 	throw unhandled (task, "PerformAttach");
     }
     TaskState processPerformDetach (Task task)
     {
+	logger.log (Level.FINE, "detach task {0}\n", task); 
 	throw unhandled (task, "PerformDetach");
     }
     TaskState processPerformStop (Task task)
     {
+	logger.log (Level.FINE, "stop task {0}\n", task); 
 	throw unhandled (task, "PerformStop");
     }
     TaskState processPerformContinue (Task task)
     {
+	logger.log (Level.FINE, "continue task {0}\n", task); 
 	throw unhandled (task, "PerformContinue");
     }
     TaskState processPerformCloned (Task task, Task clone)
     {
+	logger.log (Level.FINE, "{0} PerformCloned\n", task); 
 	throw unhandled (task, "PerformCloned");
     }
-    TaskState processPerformForked (Task task, Proc fork)
+    TaskState processPerformForked (Task task, Task fork)
     {
+	logger.log (Level.FINE, "fork task {0}\n", task); 
 	throw unhandled (task, "PerformForked");
     }
     TaskState processRequestUnblock (Task task, TaskObserver observer)
     {
+	logger.log (Level.FINE, "unblock task {0}\n", task); 
 	throw unhandled (task, "RequestUnblock");
     }
     TaskState processPerformAddObservation (Task task,
@@ -170,6 +210,7 @@ class TaskState
     protected static void processAttachedTerminated (Task task, boolean signal,
 						     int value)
     {
+	logger.log (Level.FINE, "destroy {0}\n", task); 
 	task.notifyTerminated (signal, value);
 	// A process with no tasks is dead ...?
 	if (task.proc.taskPool.size () == 0) {
@@ -186,10 +227,12 @@ class TaskState
 	{
 	    TaskState processPerformRemoval (Task task)
 	    {
+		logger.log (Level.FINE, "remove {0}\n", task); 
 		return destroyed;
 	    }
 	    TaskState processPerformAttach (Task task)
 	    {
+		logger.log (Level.FINE, "attach {0}\n", task); 
 		task.sendAttach ();
 		return attaching;
 	    }
@@ -202,6 +245,13 @@ class TaskState
 	{
 	    TaskState processPerformStopped (Task task)
 	    {
+		logger.log (Level.FINE, "{0} PerformStopped\n", task); 
+		task.proc.performTaskAttachCompleted (task);
+		return attached;
+	    }
+	    TaskState processPerformTrapped (Task task)
+	    {
+		logger.log (Level.FINE, "{0} PerformTrapped\n", task); 
 		task.proc.performTaskAttachCompleted (task);
 		return attached;
 	    }
@@ -210,6 +260,7 @@ class TaskState
 		// Outch, the task disappeared before the attach
 		// reached it, just abandon this one (but ack the
 		// operation regardless).
+		logger.log (Level.FINE, "zombie {0}\n", task); 
 		task.proc.performTaskAttachCompleted (task);
 		task.proc.remove (task);
 		return destroyed;
@@ -253,58 +304,76 @@ class TaskState
 	};
 
     /**
-     * Task just starting out, wait for it to become ready, and then
-     * let it run.
+     * Task just starting out, wait for it to both become ready, and
+     * to be unblocked, before continuing.
      */
-    private static TaskState startRunning = new TaskState ("startRunning")
+    static class Start
+	extends TaskState
+    {
+	Start (String name)
 	{
-	    TaskState processPerformStopped (Task task)
-	    {
-		task.sendSetOptions ();
-		task.sendContinue (0);
-		task.notifyAttached ();
-		return running;
+	    super ("Start." + name);
+	}
+	private static TaskState attemptAttach (Task task)
+	{
+	    task.sendSetOptions ();
+	    if (task.blockers.size () > 0) {
+		return Start.blocked;
 	    }
-	    TaskState processPerformTrapped (Task task)
+	    if (task.notifyAttached () > 0) {
+		return blockedContinue;
+	    }
+	    task.sendContinue (0);
+	    return running;
+	}
+	TaskState processPerformAddObservation (Task task,
+						Observation observation)
+	{
+	    logger.log (Level.FINE, "{0} PerformAddObservation\n",
+			task); 
+	    observation.add ();
+	    return this;
+	}
+	
+	private static TaskState waitForStop = new Start ("waitForStop")
 	    {
-		task.sendSetOptions ();
-		if (task.notifyAttached () > 0)
-		    return blockedContinue;
-		else {
-		    task.sendContinue (0);
-		    return running;
+		TaskState processRequestUnblock (Task task,
+						 TaskObserver observer)
+		{
+		    logger.log (Level.FINE, "{0} RequestUnblock\n", task); 
+		    task.blockers.remove (observer);
+		    return Start.waitForStop;
 		}
-	    }
-	    TaskState processPerformTerminating (Task task, boolean signal,
-						 int value)
+		TaskState processPerformTrapped (Task task)
+		{
+		    logger.log (Level.FINE, "{0} PerformTrapped\n", task);
+		    return attemptAttach (task);
+		}
+		TaskState processPerformStopped (Task task)
+		{
+		    logger.log (Level.FINE, "{0} PerformStopped\n", task);
+		    return attemptAttach (task);
+		}
+	    };
+	
+	private static TaskState blocked = new Start ("blocked")
 	    {
-		task.notifyTerminating (signal, value);
-		if (signal)
-		    task.sendContinue (value);
-		else
-		    task.sendContinue (0);
-		return running;
-	    }
-	    TaskState processPerformTerminated (Task task, boolean signal,
-						int value)
-	    {
-		task.proc.remove (task);
-		processAttachedTerminated (task, signal, value);
-		return destroyed;
-	    }
-	    TaskState processPerformAddObservation (Task task,
-						    Observation observation)
-	    {
-		observation.add ();
-		return startRunning;
-	    }
-	};
+		TaskState processRequestUnblock (Task task,
+						 TaskObserver observer)
+		{
+		    logger.log (Level.FINE, "{0} RequestUnblock\n", task); 
+		    task.blockers.remove (observer);
+		    return attemptAttach (task);
+		}
+	    };
+    }
 
     // A manually stopped task.
     private static TaskState stopping = new TaskState ("stopping")
 	{
 	    TaskState processRequestStop (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", task); 
 		return stopping;
 	    }
 	    TaskState processPerformSyscalled (Task task)
@@ -316,12 +385,14 @@ class TaskState
 	    TaskState processPerformStopped (Task task)
 	    {
 		// XXX: Not a standard observer.
+		logger.log (Level.FINE, "stop {0}\n", task); 
 		task.requestedStopEvent.notify (task);
 		return stopped;
 	    }
 	    TaskState processPerformTrapped (Task task)
 	    {
 		// XXX: Not a standard observer.
+		logger.log (Level.FINE, "trap {0}\n", task); 
 		task.requestedStopEvent.notify (task);
 		return paused;
 	    }
@@ -351,6 +422,7 @@ class TaskState
 	    }
 	    TaskState processPerformSignaled (Task task, int sig)
 	    {
+		logger.log (Level.FINE, "signal {0}\n", this); 
 		if (task.notifySignaled (sig) > 0) {
 		    return new BlockedSignal (sig);
 		}
@@ -361,6 +433,7 @@ class TaskState
 	    }
 	    TaskState processPerformSyscalled (Task task)
 	    {
+		logger.log (Level.FINE, "syscall {0}\n", this); 
 		task.notifySyscallEnter ();
 		task.sendContinue (0);
 		return runningInSyscall;
@@ -368,6 +441,7 @@ class TaskState
 	    TaskState processPerformTerminating (Task task, boolean signal,
 						 int value)
 	    {
+		logger.log (Level.FINE, "terminate {0}\n", this); 
 		task.notifyTerminating (signal, value);
 		if (signal)
 		    task.sendContinue (value);
@@ -378,6 +452,7 @@ class TaskState
 	    TaskState processPerformTerminated (Task task, boolean signal,
 						int value)
 	    {
+		logger.log (Level.FINE, "terminate {0}\n", this); 
 		task.proc.remove (task);
 		processAttachedTerminated (task, signal, value);
 		return destroyed;
@@ -385,6 +460,7 @@ class TaskState
 	    TaskState processPerformExeced (Task task)
 	    {
 		// Remove all tasks, retaining just this one.
+		logger.log (Level.FINE, "exec {0}\n", this); 
 		task.proc.retain (task);
 		if (task.notifyExeced () > 0) {
 		    return blockedContinue;
@@ -396,43 +472,46 @@ class TaskState
 	    }
     	    TaskState processPerformDisappeared (Task task, Throwable w)
     	    {
-		return zombied;
+		logger.log (Level.FINE, "disappeared {0}\n", this); 
+		return disappeared;
     	    }
 	    TaskState processRequestStop (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		task.sendStop ();
 		return stopping;
 	    }
 	    TaskState processRequestContinue (Task task)
 	    {
+		logger.log (Level.FINE, "continue {0}\n", this); 
 		return running;
 	    }
 	    TaskState processPerformDetach (Task task)
 	    {
+		logger.log (Level.FINE, "detach {0}\n", this); 
 		task.sendStop ();
 		return detaching;
 	    }
 	    TaskState processPerformStop (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		task.sendStop ();
 		return performingStop;
 	    }
 	    TaskState processPerformCloned (Task task, Task clone)
 	    {
-		if (task.notifyCloned (clone) > 0) {
+		logger.log (Level.FINE, "{0} PerformedClone\n", task); 
+		if (task.notifyCloned (clone) > 0)
 		    return blockedContinue;
-		}
-		else {
-		    task.sendContinue (0);
-		    return running;
-		}
+		task.sendContinue (0);
+		return running;
 	    }
-	    TaskState processPerformForked (Task task, Proc fork)
+	    TaskState processPerformForked (Task task, Task fork)
 	    {
+		logger.log (Level.FINE, "fork {0}\n", this); 
 		if (task.notifyForked (fork) > 0)
 		    return blockedContinue;
-		else
-		    task.sendContinue (0);
+		task.sendContinue (0);
 		return running;
 	    }
 	    TaskState processPerformAddObservation (Task task,
@@ -441,6 +520,13 @@ class TaskState
 		observation.add ();
 		return running;
 	    }
+	    TaskState processRequestUnblock (Task task,
+						    TaskObserver observer)
+	    {
+		// XXX: What to do about a stray unblock?
+		// observer.fail (new RuntimeException (task, "not blocked");
+		return running;
+	    }
 	};
 
     // Task is running inside a syscall.
@@ -499,6 +585,7 @@ class TaskState
 	{
 	    TaskState processPerformStopped (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		task.proc.performTaskStopCompleted (task);
 		return stopped;
 	    }
@@ -512,16 +599,11 @@ class TaskState
 	    }
 	    TaskState processPerformStopped (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		task.sendDetach (0);
 		task.proc.performTaskDetachCompleted (task);
 		return unattached;
 	    }
-	    TaskState processPerformSyscalled (Task task)
-	    {
-		task.notifySyscallEnter ();
-		task.sendContinue (0);
-		return detachingInSyscall;
-	    }
 	    TaskState processPerformTerminating (Task task, boolean signal,
 						 int value)
 	    {
@@ -532,6 +614,36 @@ class TaskState
 		task.proc.performTaskDetachCompleted (task);
 		return unattached;
 	    }
+	    TaskState processPerformForked (Task task, Task fork)
+	    {
+		logger.log (Level.FINE, "{0} PerformForked\n", this);
+		fork.sendDetach (0);
+		task.sendDetach (0);
+		task.proc.performTaskDetachCompleted (task);
+		return unattached;
+	    }
+	    TaskState processPerformCloned (Task task, Task clone)
+	    {
+		logger.log (Level.FINE, "{0} PerformCloned\n", task);
+		task.sendDetach (0);
+		// Let the proc sort out the clone.
+		task.proc.performTaskDetachCompleted (task, clone);
+		return unattached;
+	    }
+	    TaskState processPerformExeced (Task task)
+	    {
+		logger.log (Level.FINE, "{0} PerformExeced\n", this);
+		task.sendDetach (0);
+		task.proc.performTaskDetachCompleted (task);
+		return unattached;
+	    }
+	    TaskState processPerformSignaled (Task task, int signal)
+	    {
+		logger.log (Level.FINE, "{0} PerformSignaled\n", this);
+		task.sendDetach (signal);
+		task.proc.performTaskDetachCompleted (task);
+		return unattached;
+	    }
 	};
 
     private static TaskState detachingInSyscall = new TaskState ("detachingInSyscall")
@@ -551,6 +663,7 @@ class TaskState
 	    TaskState processPerformTerminating (Task task, boolean signal,
 						 int value)
 	    {
+		logger.log (Level.FINE, "terminate {0}\n", this); 
 		if (signal)
 		    task.sendDetach (value);
 		else
@@ -566,6 +679,7 @@ class TaskState
 	    {
 		// XXX: Not a standard observer.  Notify SIGTRAP
 		// indicating that the step is done.
+		logger.log (Level.FINE, "trap {0}\n", this); 
 		task.stepEvent.notify (task);
 		return stopped;
 	    }
@@ -588,18 +702,16 @@ class TaskState
 	}
 	public String toString ()
 	{
-	    return "BlockedSignal=" + sig;
+	    return "BlockedSignal,sig=" + sig;
 	}
 	TaskState processRequestUnblock (Task task, TaskObserver observer)
 	{
+	    logger.log (Level.FINE, "{0} requestUnblock\n", this); 
 	    task.blockers.remove (observer);
-	    if (task.blockers.size () == 0) {
-		task.sendContinue (sig);
-		return running;
-	    }
-	    else {
+	    if (task.blockers.size () > 0)
 		return this; // Still blocked.
-	    }
+	    task.sendContinue (sig);
+	    return running;
 	}
     }
     /**
@@ -621,15 +733,18 @@ class TaskState
 	    }
 	    TaskState processRequestStop (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		return stopped;
 	    }
 	    TaskState processRequestContinue (Task task)
 	    {
+		logger.log (Level.FINE, "continue {0}\n", this); 
 		task.sendContinue (0);
 		return running;
 	    }
 	    TaskState processRequestStepInstruction (Task task)
 	    {
+		logger.log (Level.FINE, "step insn {0}\n", this); 
 		task.sendStepInstruction (0);
 		return stepping;
 	    }
@@ -637,6 +752,7 @@ class TaskState
 	    {
 		// XXX: Need to save the stop signal so that the
 		// continue is correct.
+		logger.log (Level.FINE, "continue {0}\n", this); 
 		task.sendContinue (0);
 		task.proc.performTaskContinueCompleted (task);
 		return running;
@@ -645,6 +761,7 @@ class TaskState
 	    {
 		// XXX: Need to hang onto the signal that was about to
 		// be delivered?
+		logger.log (Level.FINE, "detach {0}\n", this); 
 		task.sendDetach (0);
 		task.proc.performTaskDetachCompleted (task);
 		return unattached;
@@ -659,10 +776,12 @@ class TaskState
 	    }
 	    TaskState processRequestStop (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		return paused;
 	    }
 	    TaskState processRequestContinue (Task task)
 	    {
+		logger.log (Level.FINE, "continue {0}\n", this); 
 		task.sendContinue (0);
 		return unpaused;
 	    }
@@ -676,10 +795,12 @@ class TaskState
 	    }
 	    TaskState processRequestContinue (Task task)
 	    {
+		logger.log (Level.FINE, "continue {0}\n", this); 
 		return unpaused;
 	    }
 	    TaskState processPerformStopped (Task task)
 	    {
+		logger.log (Level.FINE, "stop {0}\n", this); 
 		task.sendContinue (0);
 		return running;
 	    }
@@ -691,11 +812,12 @@ class TaskState
 	    }
 	};
 
-    private static TaskState zombied = new TaskState ("zombied")
+    private static TaskState disappeared = new TaskState ("disappeared")
 	{
 	    TaskState processPerformTerminated (Task task, boolean signal,
 						int value)
 	    {
+		logger.log (Level.FINE, "terminate {0}\n", this); 
 		task.proc.remove (task);
 		processAttachedTerminated (task, signal, value);
 		return destroyed;
@@ -703,16 +825,18 @@ class TaskState
 	    TaskState processPerformTerminating (Task task, boolean signal,
 						 int value)
 	    {
+		logger.log (Level.FINE, "terminate {0}\n", this); 
 		task.notifyTerminating (signal, value);
 // 		if (signal)
 // 		    task.sendContinue (value);
 // 		else
 // 		    task.sendContinue (0);
-		return zombied;
+		return disappeared;
 	    }
     	    TaskState processPerformDisappeared (Task task, Throwable w)
     	    {
-		return zombied;
+		logger.log (Level.FINE, "disappeared {0}\n", this); 
+		return disappeared;
     	    }
 	};
 
@@ -731,6 +855,7 @@ class TaskState
 		// Lie; the Proc wants to know that the operation has
 		// been processed rather than the request was
 		// successful.
+		logger.log (Level.FINE, "attach {0}\n", this); 
 		task.proc.performTaskAttachCompleted (task);
 		return destroyed;
 	    }
