@@ -68,8 +68,10 @@ public class TestLib
      * Run the event loop for a short period of time until it is
      * explicitly stopped (using EventLoop . requestStop).  During
      * this period poll for external events.
+     *
+     * XXX: Static to avoid gcc bugs.
      */
-    void assertRunUntilStop (String reason)
+    static void assertRunUntilStop (String reason)
     {
 	assertTrue ("Event loop run was explictly stopped (" + reason + ")",
 		    Manager.eventLoop.runPolling (5000));
@@ -78,16 +80,20 @@ public class TestLib
     /**
      * Process all the pending events; no polling of external events
      * is performed.
+     *
+     * XXX: Static to avoid gcc bugs.
      */
-    void runPending ()
+    static void runPending ()
     {
 	Manager.eventLoop.runPending ();
     }
 
     /**
      * Is the Proc an immediate child of PID?
+     *
+     * XXX: Static to avoid gcc bugs.
      */
-    public boolean isChildOf (int pid, Proc proc)
+    static public boolean isChildOf (int pid, Proc proc)
     {
 	// Process 1 has no parent so can't be a child of mine.
 	if (proc.getPid () == 1)
@@ -103,16 +109,20 @@ public class TestLib
      * Is the Proc an immediate child of this Proc?  Do not use
      * host.getSelf() as that, in certain situtations, can lead to
      * infinite recursion.
+     *
+     * XXX: Static to avoid gcc bugs.
      */
-    public boolean isChildOfMine (Proc proc)
+    static public boolean isChildOfMine (Proc proc)
     {
 	return isChildOf (Pid.get (), proc);
     }
 
     /**
      * Is Proc a descendant of PID?
+     *
+     * XXX: Static to avoid gcc bugs.
      */
-    public boolean isDescendantOf (int pid, Proc proc)
+    static public boolean isDescendantOf (int pid, Proc proc)
     {
 	// Climb the process tree looking for this process.
 	while (proc.getPid () > 1) {
@@ -133,12 +143,56 @@ public class TestLib
      * Is the process a descendant of this process?  Do not use
      * host.getSelf() as that, in certain situtations, can lead to
      * infinite recursion.
+     *
+     * XXX: Static to avoid gcc bugs.
      */
-    public boolean isDescendantOfMine (Proc proc)
+    static public boolean isDescendantOfMine (Proc proc)
     {
 	return isDescendantOf (Pid.get (), proc);
     }
 
+    /**
+     * Set up a handler that stops the event loop when the child sends
+     * this process an ack (indicating that it has completed a
+     * requested operation).
+     */
+    protected class AckHandler
+    {
+	AckHandler (int sig)
+	{
+	    this (new int[] { sig });
+	}
+	// NOTE: Use a different signal to thread add/del.  Within
+	// this process the signal is masked and Linux appears to
+	// propogate the mask all the way down to the exec'ed child.
+	protected final static int signal = Sig.HUP;
+	AckHandler ()
+	{
+	    this (signal);
+	}
+	private int acksRemaining;
+	AckHandler (int[] sigs)
+	{
+	    acksRemaining = sigs.length;
+	    for (int i = 0; i < sigs.length; i++) {
+		Manager.eventLoop.add (new SignalEvent (sigs[i]) {
+			public void execute ()
+			{
+			    Manager.eventLoop.requestStop ();
+			    Manager.eventLoop.remove (this);
+			    acksRemaining--;
+			}
+		    });
+	    }
+	}
+	void await ()
+	{
+	    while (acksRemaining > 0) {
+		assertRunUntilStop ("waiting for ack");
+	    }
+	}
+    }
+
     /**
      * Manage a child process.
      *
@@ -149,10 +203,6 @@ public class TestLib
      */
     protected abstract class Child
     {
-	// NOTE: Use a different signal to thread add/del.  Within
-	// this process the signal is masked and Linux appears to
-	// propogate the mask all the way down to the exec'ed child.
-	protected final static int ackSignal = Sig.HUP;
 	private int pid;
 	/**
 	 * Return the ProcessID of the child.
@@ -168,44 +218,29 @@ public class TestLib
 	{
 	    Signal.tkill (pid, sig);
 	}
-	/**
-	 * Set up a handler that stops the event loop once the child
-	 * is up and running (the child signals this using ackSignal).
-	 */
-	protected void setupAckHandler ()
-	{
-	    Manager.eventLoop.add (new SignalEvent (ackSignal)
-		{
-		    public void execute ()
-		    {
-			Manager.eventLoop.requestStop ();
-			Manager.eventLoop.remove (this);
-		    }
-		});
-	}
-	/**
-	 * Wait for the child to acknowledge the operation (using a
-	 * signal)
-	 */
-	protected void waitForAck ()
-	{
-	    assertRunUntilStop ("waiting for ack");
-	}
 	/**
 	 * Start CHILD as a running process.
 	 */
 	abstract protected int startChild (String stdin, String stdout,
 					   String stderr, String[] argv);
+	/**
+	 * Create a child process (using startChild), return once the
+	 * process is running.  Wait for acknowledge SIG.
+	 */
+	protected Child (int sig, String[] argv)
+	{
+	    AckHandler ack = new AckHandler (sig);
+	    pid = startChild (null, "/dev/null", null, argv);
+	    registerChild (pid);
+	    ack.await ();
+	}
 	/**
 	 * Create a child process (using startChild), return once the
 	 * process is running.
 	 */
 	protected Child (String[] argv)
 	{
-	    setupAckHandler ();
-	    pid = startChild (null, "/dev/null", null, argv);
-	    registerChild (pid);
-	    waitForAck ();
+	    this (AckHandler.signal, argv);
 	}
 	/**
 	 * Fudge up a Child object using PID.
@@ -295,109 +330,155 @@ public class TestLib
     }
 
     /**
-     * Create a daemon child process.
-     *
-     * Since a daemon process has process 1 as its parent, it won't
-     * lead to any wait events (or at least events that this process
-     * can see).
+     * Create an ack-process that can be manipulated using various
+     * signals (see below).
      */
-    protected class DaemonChild
+    protected abstract class AckProcess
 	extends Child
     {
+	static final int childAck = Sig.USR1;
+	static final int parentAck = Sig.USR2;
+	static final int addCloneSig = Sig.USR1;
+	static final int delCloneSig = Sig.USR2;
+	static final int addForkSig = Sig.HUP;
+	static final int delForkSig = Sig.INT;
+	static final int zombieForkSig = Sig.URG;
+	static final int execSig = Sig.PWR;
+	AckProcess (int ack, String[] argv)
+	{
+	    super (ack, argv);
+	}
+	AckProcess ()
+	{
+	    this (childAck, new String[]
+		{
+		    "./prog/kill/child",
+		    "20",
+		    // Use getpid as this testsuite always runs the
+		    // event loop from the main thread (which has
+		    // tid==pid).
+		    Integer.toString (Pid.get ()),
+		});
+	}
+	AckProcess (int count)
+	{
+	    this ();
+	    for (int i = 0; i < count; i++)
+		addClone ();
+	}
+	/** . */
+	private void spawn (int sig)
+	{
+	    AckHandler ack = new AckHandler (new int[]
+		{
+		    childAck, parentAck
+		});
+	    signal (sig);
+	    ack.await ();
+	}
+	/** Add a Task.  */
+	public void addClone ()
+	{
+	    spawn (addCloneSig);
+	}
+	/** Delete a Task.  */
+	public void delClone ()
+	{
+	    AckHandler ack = new AckHandler (parentAck);
+	    signal (delCloneSig);
+	    ack.await ();
+	}
+	/** Add a child Proc.  */
+	public void addFork ()
+	{
+	    spawn (addForkSig);
+	}
+	/** Delete a child Proc.  */
+	public void delFork ()
+	{
+	    AckHandler ack = new AckHandler (parentAck);
+	    signal (delForkSig);
+	    ack.await ();
+	}
+	/** Terminate a fork Proc (creates zombie).  */
+	public void zombieFork ()
+	{
+	    AckHandler ack = new AckHandler (parentAck);
+	    signal (zombieForkSig);
+	    ack.await ();
+	}
+	/**
+	 * Kill the parent, expect an ack from the child (there had
+	 * better be a child).
+	 */
+	void fryParent ()
+	{
+	    AckHandler ack = new AckHandler (childAck);
+	    signal (Sig.KILL);
+	    ack.await ();
+	}
+	/**
+	 * Perform an exec.
+	 */
+	void exec ()
+	{
+	    AckHandler ack = new AckHandler (childAck);
+	    signal (execSig);
+	    ack.await ();
+	}
+    }
+
+    /**
+     * Create an ack daemon.  An ack daemon has process 1, and not
+     * this process, as its parent.
+     *
+     * Since this a daemon, this process won't have to contend with
+     * its exit status - it will go to process 1.
+     */
+    protected class AckDaemonProcess
+	extends AckProcess
+    {
+	/**
+	 * Create the process as a daemon.
+	 */
 	protected int startChild (String stdin, String stdout, String stderr,
 				  String[] argv)
 	{
 	    return Fork.daemon (stdin, stdout, stderr, argv);
 	}
-	/**
-	 * Create a daemon child running ARGV.
-	 */
-	DaemonChild (String[] argv)
+	AckDaemonProcess (int ack, String[] argv)
 	{
-	    super (argv);
+	    super (ack, argv);
 	}
-	/**
-	 * Create a daemon process (one that's parent has exited
-	 * causing it to have process one as the parent).  Also create
-	 * CLONES tasks and possibly use POLLING.
-	 */
-	DaemonChild (int clones, boolean polling)
+	AckDaemonProcess ()
 	{
-	    this (new String[] {
-		      "./prog/kill/detach",
-		      Integer.toString (Pid.get ()),
-		      Integer.toString (ackSignal),
-		      "20",
-		      Integer.toString (clones),
-		      "0",
-		      polling ? "1" : "0"
-		  });
+	    super ();
 	}
-	/**
-	 * Create a daemon child process that includes CLONES
-	 * threads.  Block until both the process and all threads have
-	 * been started.
-	 */
-	DaemonChild (int clones)
+	AckDaemonProcess (int count)
 	{
-	    this (clones, false);
-	}
-	/**
-	 * Create a daemon child process.  Block until the process
-	 * has started.
-	 */
-	DaemonChild ()
-	{
-	    this (0);
-	}
-	/**
-	 * Ask the daemon process to add another task.  Run the
-	 * eventLoop until the daemon acknowledges that it has
-	 * completed the operation.
-	 */
-	void addTask ()
-	{
-	    setupAckHandler ();
-	    // Sig.USR2 indicates add a thread.
-	    signal (Sig.USR1);
-	    waitForAck ();
-	}
-	/**
-	 * Ask the daemon process to Delete a task.  Run the eventLoop
-	 * until the child acknowledges it has completed the
-	 * operation.
-	 */
-	void delTask ()
-	{
-	    setupAckHandler ();
-	    // Sig.USR2 indicates drop a thread.
-	    signal (Sig.USR2);
-	    waitForAck ();
+	    super (count);
 	}
     }
 
+
     /**
-     * Create a detached child process.
+     * Create a detached child ack process.
      *
      * Since the created process is a direct child of this process,
      * this process will see a wait event when this exits.  It is most
      * useful when a controlled process exit is required (see reap).
      */
-    protected class DetachedChild
-	extends DaemonChild
+    protected class DetachedAckProcess
+	extends AckProcess
     {
+	/**
+	 * Create a detached process that is a child of this one.
+	 */
 	protected int startChild (String stdin, String stdout, String stderr,
 				  String[] argv)
 	{
 	    return Fork.exec (stdin, stdout, stderr, argv);
 	}
-	/**
-	 * Create a child process with no threads.
-	 */
-	DetachedChild ()
-	{
-	    super ();
-	}
 	/**
 	 * Reap the child.
 	 *
@@ -430,11 +511,22 @@ public class TestLib
     }
 
     /**
-     * Create an attached child process.
+     * Create an attached child ack process.
      */
-    protected class AttachedChild
-	extends DaemonChild
+    protected class AttachedAckProcess
+	extends AckProcess
     {
+	AttachedAckProcess ()
+	{
+	    super ();
+	}
+	AttachedAckProcess (int count)
+	{
+	    super (count);
+	}
+	/**
+	 * Create the process as an attached child.
+	 */
 	protected int startChild (String stdin, String stdout, String stderr,
 				  String[] argv)
 	{
@@ -463,85 +555,6 @@ public class TestLib
 	    // Return that captured PID.
 	    return pidObserver.pid;
 	}
-	AttachedChild ()
-	{
-	    super ();
-	}
-	AttachedChild (int clones)
-	{
-	    super (clones);
-	}
-	AttachedChild (int clones, boolean polling)
-	{
-	    super (clones, polling);
-	}
-    }
-
-    /**
-     * Create a daemon process that, on demand, creates a child
-     * process that can, when requested, be turned into a zombie.
-     */
-    protected class ZombieChild
-	extends Child
-    {
-	/**
-	 * Create a child process as per request from super-object.
-	 */
-	protected int startChild (String stdin, String stdout, String stderr,
-				  String[] argv)
-	{
-	    return Fork.daemon (stdin, stdout, stderr, argv);
-	}
-	/**
-	 * Create a zombied child process.
-	 */
-	ZombieChild ()
-	{
-	    super (new String[] {
-		       "./prog/kill/zombie",
-		       Integer.toString (Pid.get ()),
-		       Integer.toString (ackSignal),
-		       "20"
-		   });
-	}
-	/**
-	 * Sanity check, no sense in trying to do a kill when there is
-	 * no child.
-	 */
-	private boolean hasChild = false;
-	/**
-	 * Tell the zombie maker to create one child.
-	 */
-	void addChild ()
-	{
-	    setupAckHandler ();
-	    signal (Sig.USR1);
-	    waitForAck ();
-	    hasChild = true;
-	}
-	/**
-	 * Tell the zombie maker to kill it's child turning it into a
-	 * zombie.
-	 */
-	void fryChild ()
-	{
-	    assertTrue ("Zombie has a child", hasChild);
-	    setupAckHandler ();
-	    signal (Sig.USR2);
-	    waitForAck ();
-	    hasChild = false;
-	}
-	/**
-	 * Kill the parent, expect an ack from the child (there had
-	 * better be a child).
-	 */
-	void fryParent ()
-	{
-	    assertTrue ("Zombie has a child", hasChild);
-	    setupAckHandler ();
-	    signal (Sig.KILL);
-	    waitForAck ();
-	}
     }
 
     /**
@@ -550,7 +563,7 @@ public class TestLib
      * In addition to methods for managing the set, there is a method
      * for unblocking all members.
      */
-    class TaskSet
+    static class TaskSet
     {
 	/**
 	 * Set of tasks being managed.
@@ -613,9 +626,8 @@ public class TestLib
 	 * Task's observer set.
 	 */
 	int addedCount;
-	public void added (Throwable e)
+	public void addedTo (Object o)
 	{
-	    assertNull ("Observer successfully added", e);
 	    addedCount++;
 	}
 	/**
@@ -623,10 +635,17 @@ public class TestLib
 	 * Task's observer set.
 	 */
 	int deletedCount;
-	public void deleted ()
+	public void deletedFrom (Object o)
 	{
 	    deletedCount++;
 	}
+	/**
+	 * The add operation failed, should never happen.
+	 */
+	public void addFailed (Object o, Throwable w)
+	{
+	    fail ("add to " + o + " failed");
+	}
     }
 
     /**
@@ -862,7 +881,7 @@ public class TestLib
      * computed value, and the number of resursive calls required to
      * compute that value.
      */
-    protected class Fibonacci
+    static protected class Fibonacci
     {
 	int callCount;
 	int value;
