@@ -60,8 +60,9 @@ import frysk.gui.srcwin.PreferenceConstants.Comments;
 import frysk.gui.srcwin.PreferenceConstants.CurrentLine;
 import frysk.gui.srcwin.PreferenceConstants.Functions;
 import frysk.gui.srcwin.PreferenceConstants.GlobalVariables;
-import frysk.gui.srcwin.PreferenceConstants.Inline;
 import frysk.gui.srcwin.PreferenceConstants.Keywords;
+import frysk.gui.srcwin.PreferenceConstants.OptimizedVariables;
+import frysk.gui.srcwin.PreferenceConstants.OutOfScopeVariables;
 import frysk.gui.srcwin.PreferenceConstants.Search;
 import frysk.gui.srcwin.PreferenceConstants.Variables;
 import frysk.gui.srcwin.cparser.CDTParser;
@@ -83,16 +84,18 @@ import frysk.gui.srcwin.dom.DOMTagTypes;
  */
 public class SourceBuffer extends TextBuffer {
 	
-	private static final String CLASS_TAG = "CLASS";
+	protected static final String CLASS_TAG = "CLASS";
 	/* CONSTANTS */
-	private static final String INLINE_TAG = "INLINE";
-	private static final String COMMENT_TAG = "COMMENT";
-	private static final String MEMBER_TAG = "MEMBER";
-	private static final String FUNCTION_TAG = "FUNCTION";
-	private static final String ID_TAG = "ID";
-	private static final String KEYWORD_TAG = "TYPE";
-	private static final String CURRENT_LINE = "currentLine";
-	private static final String FOUND_TEXT = "foundText";
+	protected static final String INLINE_TAG = "INLINE";
+	protected static final String COMMENT_TAG = "COMMENT";
+	protected static final String MEMBER_TAG = "MEMBER";
+	protected static final String FUNCTION_TAG = "FUNCTION";
+	protected static final String ID_TAG = "ID";
+	protected static final String OPTIMIZED_VAR_TAG = "OPTIMIZED_VAR";
+	protected static final String OUT_OF_SCOPE_VAR_TAG = "OOS_VAR";
+	protected static final String KEYWORD_TAG = "TYPE";
+	protected static final String CURRENT_LINE = "currentLine";
+	protected static final String FOUND_TEXT = "foundText";
 	/* END CONSTANTS */
 	
 	private Vector functions;
@@ -111,24 +114,28 @@ public class SourceBuffer extends TextBuffer {
 	private TextTag keywordTag;
 	private TextTag commentTag;
 	private TextTag classTag;
-	
-	private TextTag inlinedTag;
+	private TextTag optimizedVarTag;
+	private TextTag oosVarTag;
 		
 	private StaticParser staticParser;
 	
 	// Since conceptually each sourcebuffer will only be viewing one file, we don't
 	// need any information higher than this
-	private StackLevel scope;
+	protected StackLevel scope;
 	
 	/**
 	 * Creates a new SourceBuffer
 	 */
-	public SourceBuffer(StackLevel data){ 
+	public SourceBuffer(){ 
 		super();
-		this.scope = data;
 		this.init();
 	}
 	
+	public SourceBuffer(StackLevel scope){
+		this();
+		this.setScope(scope);
+	}
+	
 	/**
 	 * Returns true if the given line is executable.
 	 * 
@@ -170,6 +177,9 @@ public class SourceBuffer extends TextBuffer {
 	 * does not correspond to a valid line number
 	 */
 	public boolean toggleBreakpoint (int lineNum){
+        if(!this.isLineExecutable(lineNum))
+            return false;
+        
 		DOMLine line = this.scope.getData().getLine(lineNum + 1);
 		if(line == null)
 			return false;
@@ -199,8 +209,7 @@ public class SourceBuffer extends TextBuffer {
 	 * @param endCol The offset (wrt. the start of the line) that the
 	 * 		instruction ends on
 	 */
-	private void setCurrentLine(int startLine, int startCol, int endLine, int endCol){
-		System.out.println(startLine +","+ startCol +","+ endLine +","+ endCol);
+	protected void setCurrentLine(int startLine, int startCol, int endLine, int endCol){
 		
 		this.startCurrentLine = this.createMark("currentLineStart", 
 				this.getIter(this.getLineIter(startLine-1).getOffset() + startCol),
@@ -208,7 +217,7 @@ public class SourceBuffer extends TextBuffer {
 		if(endCol != StackLevel.EOL){
 			this.endCurrentLine = this.createMark("currentLineEnd", 
 				this.getIter(this.getLineIter(endLine-1).getOffset() + endCol),
-				true);
+				false);
 		}
 		else{
 			TextIter lineStart = this.getLineIter(endLine - 1);
@@ -253,6 +262,23 @@ public class SourceBuffer extends TextBuffer {
 		this.variableTag.setWeight(Weight.intern(weight));
 		this.updateTagStyle(currentNode, Variables.ITALICS, Style.NORMAL, this.variableTag);
 
+		// optimized out variable syntax highlighting
+		this.updateTagColor(currentNode, OptimizedVariables.COLOR_PREFIX,
+				OptimizedVariables.DEFAULT, this.optimizedVarTag, true);
+		this.updateTagStyle(currentNode, OptimizedVariables.ITALICS, Style.NORMAL,
+				this.optimizedVarTag);
+		boolean strikethrough = currentNode.getBoolean(OptimizedVariables.STRIKETHROUGH, 
+				OptimizedVariables.STRIKETHROUGH_DEFAULT);
+		this.optimizedVarTag.setStrikethrough(strikethrough);
+		
+		// out of scope variable syntax highlighting
+		this.updateTagColor(currentNode, OutOfScopeVariables.COLOR_PREFIX,
+				OutOfScopeVariables.DEFAUT, this.oosVarTag, true);
+		this.updateTagStyle(currentNode, OutOfScopeVariables.ITALICS, Style.NORMAL,
+				this.oosVarTag);
+		weight = currentNode.getInt(OutOfScopeVariables.WEIGHT, Weight.NORMAL.getValue());
+		this.oosVarTag.setWeight(Weight.intern(weight));
+		
 		// Global variable syntax highlighting
 		this.updateTagColor(currentNode, GlobalVariables.COLOR_PREFIX, GlobalVariables.DEFAULT,
 				this.globalTag, true);
@@ -277,9 +303,6 @@ public class SourceBuffer extends TextBuffer {
 		weight = currentNode.getInt(Classes.WEIGHT, Weight.NORMAL.getValue());
 		this.classTag.setWeight(Weight.intern(weight));
 		this.updateTagStyle(currentNode, Classes.ITALICS, Style.NORMAL, this.classTag);
-		
-		// Inlined tag background
-		this.updateTagColor(currentNode, Inline.COLOR_PREFIX, Inline.DEFAULT, this.inlinedTag, false);
 	}
 	
 	/**
@@ -530,9 +553,18 @@ public class SourceBuffer extends TextBuffer {
 	 */
 	public void addVariable(int offset, int length){
 		DOMLine line = this.scope.getData().getLine(this.getIter(offset).getLineNumber() + 1);
-		line.addTag(DOMTagTypes.LOCAL_VAR,
+		if(line.getLineNum() == 15)
+			line.addTag(DOMTagTypes.LOCAL_VAR,
 				this.getText(this.getIter(offset), this.getIter(offset+length), true),
 				this.getIter(offset).getLineOffset());
+		else if(line.getLineNum() < 15)
+			line.addTag(DOMTagTypes.OUT_OF_SCOPE_VAR,
+					this.getText(this.getIter(offset), this.getIter(offset+length), true),
+					this.getIter(offset).getLineOffset());
+		else
+			line.addTag(DOMTagTypes.OPTIMIZED_VAR	,
+					this.getText(this.getIter(offset), this.getIter(offset+length), true),
+					this.getIter(offset).getLineOffset());
 	}
 	
 	public void addVariable(int lineNum, int lineOffset, int length){
@@ -600,14 +632,26 @@ public class SourceBuffer extends TextBuffer {
 	 * @param includeInlindes Whether to include inlined code in the line count
 	 * @return The number of lines in the file
 	 */
-	public int getLineCount(boolean includeInlindes){
+	public int getLineCount(){
 		return this.scope.getData().getLineCount();
 	}
+    
+    public int getLastLine(){
+        return this.getLineCount();
+    }
 	
 	public boolean hasInlineCode(int lineNumber){
 		return this.scope.getData().getLine(lineNumber+1).hasInlinedCode();
 	}
 	
+	public DOMInlineInstance getInlineInstance(int lineNumber){
+		Iterator iter = this.scope.getData().getLine(lineNumber+1).getInlines();
+		if(!iter.hasNext())
+			return null;
+		
+		return new DOMInlineInstance((Element) iter.next()); 
+	}
+	
 	public void setScope(StackLevel scope){
 		for(int i = 0; i < functions.size(); i++)
 			this.deleteMark(((String) functions.get(i)));
@@ -644,20 +688,9 @@ public class SourceBuffer extends TextBuffer {
 		this.keywordTag = this.createTag(KEYWORD_TAG);
 		this.globalTag = this.createTag(MEMBER_TAG);
 		this.commentTag = this.createTag(COMMENT_TAG);
-		this.classTag = this.createTag(CLASS_TAG);	
-		
-		this.inlinedTag = this.createTag(INLINE_TAG);
-		
-		try {
-			this.loadFile();
-		} catch (Exception e) {
-			e.printStackTrace();
-		}
-		
-		this.setCurrentLine(this.scope.getStartingLineNum(),
-				this.scope.getColStart(),
-				this.scope.getEndLine(),
-				this.scope.getColEnd());
+		this.classTag = this.createTag(CLASS_TAG);
+		this.optimizedVarTag = this.createTag(OPTIMIZED_VAR_TAG);
+		this.oosVarTag = this.createTag(OUT_OF_SCOPE_VAR_TAG);
 	}
 	
 	/**
@@ -668,25 +701,16 @@ public class SourceBuffer extends TextBuffer {
 	 * @throws FileNotFoundException
 	 * @throws JGException
 	 */
-	private void loadFile() throws FileNotFoundException, JGException{
+	protected void loadFile() throws FileNotFoundException, JGException{
 		Iterator lines = this.scope.getData().getLines();
 		
-		String bufferText = "";
-		
-		// First get the text, append it all together, and add it to ourselves
-		while(lines.hasNext()){
-			DOMLine line = new DOMLine((Element) lines.next());
-			
-			if(line.getText().equals(""))
-				bufferText += "\n";
-			else
-				bufferText += line.getText();
-		}
+		String bufferText = loadLines(lines);
 		
 		this.deleteText(this.getStartIter(), this.getEndIter());
 		this.insertText(bufferText);
 		
 		if(!this.scope.isParsed()){
+            
 			// now pass the resulting text to the parser
 			if(this.staticParser == null)
 				this.staticParser = new CDTParser();
@@ -700,11 +724,30 @@ public class SourceBuffer extends TextBuffer {
 		
 		this.createTags();
 	}
+
+	/**
+	 * @param lines Iterator of the lines to store in this SourceBuffer
+	 * @return What should be the contents of the buffer
+	 */
+	protected String loadLines(Iterator lines) {
+		String bufferText = "";
+		
+		// First get the text, append it all together, and add it to ourselves
+		while(lines.hasNext()){
+			DOMLine line = new DOMLine((Element) lines.next());
+			
+			if(line.getText().equals(""))
+				bufferText += "\n";
+			else
+				bufferText += line.getText();
+		}
+		return bufferText;
+	}
 	
 	/*
 	 * Reads through the DOM and creates all the tags necessary
 	 */
-	private void createTags(){
+	protected void createTags(){
 		Iterator lines = this.scope.getData().getLines();
 		
 		// Iterate through all the lines
@@ -732,6 +775,18 @@ public class SourceBuffer extends TextBuffer {
 							this.getIter(lineOffset + tag.getStart() + tag.getLength()));
 				}
 				
+				else if(type.equals(DOMTagTypes.OPTIMIZED_VAR)){
+					this.applyTag(OPTIMIZED_VAR_TAG, 
+							this.getIter(lineOffset + tag.getStart()),
+							this.getIter(lineOffset + tag.getStart() + tag.getLength()));
+				}
+				
+				else if(type.equals(DOMTagTypes.OUT_OF_SCOPE_VAR)){
+					this.applyTag(OUT_OF_SCOPE_VAR_TAG, 
+							this.getIter(lineOffset + tag.getStart()),
+							this.getIter(lineOffset + tag.getStart() + tag.getLength()));
+				}
+				
 				else if(type.equals(DOMTagTypes.CLASS_DECL)){
 					this.applyTag(SourceBuffer.CLASS_TAG, 
 							this.getIter(lineOffset + tag.getStart()),
@@ -810,4 +865,8 @@ public class SourceBuffer extends TextBuffer {
 		tag.setStyle(Style.intern(style));
 	}
 
+	public StackLevel getScope() {
+		return scope;
+	}
+
 }
