@@ -63,6 +63,7 @@ import org.gnu.gtk.event.MouseEvent;
 import org.gnu.gtk.event.MouseListener;
 import org.gnu.pango.Alignment;
 import org.gnu.pango.Layout;
+import org.jdom.Element;
 
 import frysk.gui.srcwin.PreferenceConstants.Background;
 import frysk.gui.srcwin.PreferenceConstants.CurrentLine;
@@ -70,6 +71,9 @@ import frysk.gui.srcwin.PreferenceConstants.ExecMarks;
 import frysk.gui.srcwin.PreferenceConstants.LineNumbers;
 import frysk.gui.srcwin.PreferenceConstants.Margin;
 import frysk.gui.srcwin.PreferenceConstants.Text;
+import frysk.gui.srcwin.dom.DOMImage;
+import frysk.gui.srcwin.dom.DOMInlineInstance;
+import frysk.gui.srcwin.dom.DOMSource;
 
 /** 
  * This class is used to add some functionality to TextView that may be needed
@@ -96,10 +100,11 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 	// How far to start writing breakpoints, etc. from the left side of the margin
 	protected int marginWriteOffset;
 	
-	private TextChildAnchor anchor;
+	protected TextChildAnchor anchor;
 	private SourceWindow parent;
 	
 	protected boolean expanded = false;
+
 	/**
 	 * Constructs a new SourceViewWidget. If you don't specify a buffer before using it,
 	 * a default one will be created for you.
@@ -108,9 +113,13 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 	 * @param scope The source file that this widget will be displaying
 	 */
 	public SourceViewWidget(Preferences parentPrefs, StackLevel scope, SourceWindow parent) {
+		this(parentPrefs, new SourceBuffer(scope), parent);
+	}
+	
+	public SourceViewWidget(Preferences parentPrefs, SourceBuffer buffer, SourceWindow parent){
 		super(gtk_text_view_new());
 		this.parent = parent;
-		this.buf = new SourceBuffer(scope);
+		this.buf = buffer;
 		this.setBuffer(this.buf);
 		this.topPrefs = parentPrefs;
 		this.lnfPrefs = parentPrefs.node(PreferenceConstants.LNF_NODE);
@@ -138,7 +147,7 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 		// Sidebar
 		if(this.lnfPrefs.getBoolean(LineNumbers.SHOW, true)){
 			Layout lo = new Layout(this.getContext());
-			lo.setText(""+(this.buf.getLineCount()+1));
+			lo.setText(""+(this.buf.getLastLine()+1));
 			this.marginWriteOffset = lo.getPixelWidth();
 		}
 		else{
@@ -236,11 +245,26 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 			Point p = this.windowToBufferCoords(TextWindowType.TEXT, 0, y);
 			
 			TextIter iter = this.getIterAtLocation(p.getX(), p.getY());
-			final int lineNum = iter.getLineNumber();
+			
+			int theLine = iter.getLineNumber();
+			boolean overNested = false;
+			
+			// We want to ignore mouse clicks in the margin next to 
+			// expanded inline code
+			if(theLine == this.buf.getCurrentLine() + 1 && expanded)
+				return false;
+			
+			if(theLine > this.buf.getCurrentLine() && expanded){
+				theLine--;
+				overNested = true;
+			}
+			
+			final int lineNum = theLine;
 			
 			// only popup a window if the line is executable
 			if(event.getButtonPressed() == MouseEvent.BUTTON3 &&
-					this.buf.isLineExecutable(lineNum)){
+					this.buf.isLineExecutable(lineNum) &&
+					(!expanded || overNested)){
 				Menu m = new Menu();
 				MenuItem mi = new MenuItem("Breakpoint information...", false);
 				mi.addListener(new MenuItemListener() {
@@ -271,7 +295,8 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 			
 			// Left click in the margin for a line with inline code - toggle the display of it
 			if(event.getButtonPressed() == MouseEvent.BUTTON1 &&
-					lineNum == this.buf.getCurrentLine()){
+					lineNum == this.buf.getCurrentLine() &&
+					this.buf.hasInlineCode(lineNum)){
 				this.toggleChild();
 			}
 		}
@@ -337,20 +362,20 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 	}
 	
 	public void setSubscopeAtCurrentLine(SourceViewWidget child){
-		TextIter line = buf.getLineIter(buf.getCurrentLine()+1);
+		TextIter line = buf.getLineIter(buf.getCurrentLine() + 1);
 		
 		if(anchor != null)
 			buf.deleteText(line, buf.getIter(line.getOffset()+1));
 		else
 			buf.insertText(line, "\n");
-		this.anchor = buf.createChildAnchor(buf.getLineIter(buf.getCurrentLine()+1));
+		this.anchor = buf.createChildAnchor(buf.getLineIter(buf.getCurrentLine() + 1));
 		
 		this.expanded = true;
 		this.addChild(child, anchor);
 	}
 	
 	public void clearSubscopeAtCurrentLine(){
-		TextIter line = buf.getLineIter(buf.getCurrentLine()+1);
+		TextIter line = buf.getLineIter(buf.getCurrentLine() + 1);
 		buf.deleteText(line, buf.getIter(line.getOffset()+2));
 		this.expanded = false;
 		this.anchor = null;
@@ -381,14 +406,24 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 
 	public void toggleChild() {
 		if(!expanded){
-			InlineViewer nested = new InlineViewer(this.topPrefs, this.parent);
+			expanded = true;
+			
+			DOMInlineInstance instance = this.buf.getInlineInstance(this.buf.getCurrentLine());
+			
+			Element node = instance.getElement();
+			while(!node.getName().equals(DOMImage.IMAGE_NODE))
+				node = node.getParentElement();
+			
+			DOMSource scope = new DOMImage(node).getSource(instance.getDeclaration().getSourceName());
+			InlineViewer nested = new InlineViewer(this.topPrefs, this.parent, 
+					scope, instance);
 			this.setSubscopeAtCurrentLine(nested);
 		}
 		else{
+			expanded = false;
 			this.clearSubscopeAtCurrentLine();
 		}
 			
-		expanded = !expanded;
 	}
 	
 	/*
@@ -441,39 +476,46 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 		int markB = this.lnfPrefs.getInt(ExecMarks.COLOR_PREFIX+"B", ExecMarks.DEFAULT.getBlue());
 		
 		int currentHeight = 0;		
-		int actualIndex = 0;
-		int totalInlinedLines = 0;
+		int actualIndex = firstLine;
+		boolean skipNextLine = false;
 		
 		int drawingHeight = 0;
+        int lineHeight = 0;
 		int gapHeight = 0;
 		
 		// If the refresh is starting after the current line, we have to add that offset in to
 		// make sure the gap in line numbers is maintained
 		if(expanded && firstLine > this.buf.getCurrentLine())
-			gapHeight = this.getLineYRange(this.getBuffer().getLineIter(this.buf.getCurrentLine()+1)).getHeight();
+			gapHeight = this.getLineYRange(this.getBuffer().getLineIter(this.buf.getCurrentLine() + 1)).getHeight();
 		
-		for(int i = firstLine; i <= lastLine && i < this.buf.getLineCount(false); i++){
-		
-			if(i > this.buf.getCurrentLine())
+		for(int i = firstLine; i <= lastLine && i < this.buf.getLineCount(); i++){
+			
+			if(i > this.buf.getCurrentLine()){
 				drawingHeight = currentHeight + gapHeight;
-			else
+                if(expanded)
+                    lineHeight = this.getLineYRange(this.getBuffer().getLineIter(i+1)).getHeight();
+                else
+                    lineHeight = this.getLineYRange(this.getBuffer().getLineIter(i)).getHeight();
+            }
+			else{
 				drawingHeight = currentHeight;
+                lineHeight = this.getLineYRange(this.getBuffer().getLineIter(i)).getHeight();
+            }
 			
-			// get the current line height, etc.
-			int lineHeight = this.getLineYRange(this.getBuffer().getLineIter(i)).getHeight();
 			int iconStart = lineHeight/2;
 			
-			if(totalInlinedLines == 1){
-				totalInlinedLines = 0;
-			
+			if(skipNextLine){
+				skipNextLine = false;
+				
 				gapHeight = this.getLineYRange(this.getBuffer().getLineIter(actualIndex++)).getHeight();
- 
+				
 				i--;
 				continue;
 			}
 			
-			// For the current line, draw background using the currentLine color
+			// For the current line, do some special stuff
 			if(i == this.buf.getCurrentLine()){
+					
 				context.setRGBForeground(new Color(lineR, lineG, lineB));
 				if(showMarks)
 					drawingArea.drawRectangle(context, true, 0, actualFirstStart+drawingHeight, 
@@ -482,6 +524,20 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 					drawingArea.drawRectangle(context, true, 0, actualFirstStart+drawingHeight, 
 							this.marginWriteOffset+20, lineHeight);
 				context.setRGBForeground(new Color(r,g,b));
+				
+				if(this.buf.hasInlineCode(i)){
+					context.setRGBForeground(new Color(markR,markG,markB));
+					Layout lo = this.createLayout("i");
+					lo.setAlignment(Alignment.RIGHT);
+					if(showMarks)
+						drawingArea.drawLayout(context, this.marginWriteOffset+25, actualFirstStart+drawingHeight, lo);
+					else
+						drawingArea.drawLayout(context, this.marginWriteOffset+5, actualFirstStart+drawingHeight, lo);
+					context.setRGBForeground(new Color(r,g,b));
+					
+					if(this.expanded)
+						skipNextLine = true;
+				}
 			}
 			
 			
@@ -493,49 +549,30 @@ public class SourceViewWidget extends TextView implements ExposeListener, MouseL
 				context.setRGBForeground(new Color(r,g,b));
 			}
 			
-			if(i == this.buf.getCurrentLine() && this.buf.hasInlineCode(i)){
-				System.out.println("Found inline code on current line");
-				context.setRGBForeground(new Color(markR,markG,markB));
-				Layout lo = this.createLayout("i");
-//				Layout lo = new Layout(this.getContext());
-				lo.setAlignment(Alignment.RIGHT);
-//				lo.setText("i");
-				if(showMarks)
-					drawingArea.drawLayout(context, this.marginWriteOffset+25, actualFirstStart+drawingHeight, lo);
-				else
-					drawingArea.drawLayout(context, this.marginWriteOffset+5, actualFirstStart+drawingHeight, lo);
-				context.setRGBForeground(new Color(r,g,b));
-				
-				if(this.expanded)
-					totalInlinedLines = 1;
-				else
-					totalInlinedLines = 0;
-			}
-			
 			// Draw line numbers
-			if(showLines){
-				Layout lo = this.createLayout(""+(i+1));
-//				Layout lo = new Layout(this.getContext());
-				lo.setAlignment(Alignment.RIGHT);
-				lo.setWidth(this.marginWriteOffset);
-//				lo.setText(""+(i+1));
-				
-				drawingArea.drawLayout(context, this.marginWriteOffset, actualFirstStart+drawingHeight, lo);
-			}
+			if(showLines)
+				drawLineNumber(drawingArea, context, actualFirstStart + drawingHeight, i);
 			
 			// draw breakpoints
 			if(this.buf.isLineBroken(i)){
 				int iconHeight = lineHeight - 8;
 				
 				context.setRGBForeground(new Color(65535,0,0));
-				drawingArea.drawRectangle(context, true, this.marginWriteOffset+5, actualFirstStart+currentHeight+4, iconHeight, iconHeight);
+				drawingArea.drawRectangle(context, true, this.marginWriteOffset+5, actualFirstStart+drawingHeight+4, iconHeight, iconHeight);
 				context.setRGBForeground(new Color(r,g,b));
 			}
 			
 			// update height for next line
 			currentHeight += this.getLineYRange(this.getBuffer().getLineIter(actualIndex++)).getHeight();
 		}
-		
 	}
+
+    protected void drawLineNumber(Window drawingArea, GC context, int drawingHeight, int number) {
+        Layout lo = this.createLayout(""+(number+1));
+        lo.setAlignment(Alignment.RIGHT);
+        lo.setWidth(this.marginWriteOffset);
+        
+        drawingArea.drawLayout(context, this.marginWriteOffset, drawingHeight, lo);
+    }
 	
 }
