@@ -66,7 +66,7 @@ public class DOMTestDOMFrysk {
 //
 //	private static String[] inline_funcs = { "do_something", "b", "f" };
 //
-	private static boolean[] is_inline = { false, true, false };
+//	private static boolean[] is_inline = { false, true, false };
 //
 //	private static String[] do_something = { "void do_something(){", "   b();",
 //			"}" };
@@ -370,7 +370,7 @@ public class DOMTestDOMFrysk {
 		BigInteger pc = BigInteger.valueOf(25842);
 		for (int ctr = 0; ctr < main_prog.length; ctr++) {
 			testDOMSource.addLine(ctr + 1, main_prog[ctr], true,
-					is_inline[ctr], false, offset_index[ctr], pc);
+					false, offset_index[ctr], pc);
 			pc = pc.add(no_bytes);
 		}
 
@@ -489,7 +489,7 @@ public class DOMTestDOMFrysk {
 		int start_inline = main_prog[1].indexOf(test_inline) + main_prog[0].length();
 		int end_inline = main_prog[1].indexOf(test_inline) + test_inline.length() +
 				main_prog[0].length();
-		testDOMLine.addInlineInst(test_inline, start_inline, end_inline);
+		testDOMLine.addInlineInst(test_inline, start_inline, end_inline, 0);
 		if (testDOMLine.getInlineElement(test_inline).
 				getAttributeValue(DOMInlineInstance.LINEINST_ATTR)
 				== test_inline) {
@@ -515,24 +515,24 @@ public class DOMTestDOMFrysk {
 			}
 		}
 		
-		Iterator iter_getinlines = testDOMLine.getInlines();
-		while (iter_getinlines.hasNext()) {
-			Element getinlines = (Element) iter_getinlines.next();
-			String inline_name = getinlines.getAttributeValue
-				(DOMInlineInstance.LINEINST_ATTR);
-			if (inline_name
-					== test_inline) {
-				System.out.println("passed...DOMLine.getInlines");
-			} else {
-				System.out.println("failed...DOMLine.getInlines");
-			}
-			DOMFunction test_func = testDOMImage.getFunction(inline_name);
-			if (test_func.getName() == test_inline) {
-				System.out.println("passed...DOMFunction.getFunction");
-			} else {
-				System.out.println("failed...DOMFunction.getFunction");
-			}
-		}
+//		Iterator iter_getinlines = testDOMLine.getInlines();
+//		while (iter_getinlines.hasNext()) {
+//			Element getinlines = (Element) iter_getinlines.next();
+//			String inline_name = getinlines.getAttributeValue
+//				(DOMInlineInstance.LINEINST_ATTR);
+//			if (inline_name
+//					== test_inline) {
+//				System.out.println("passed...DOMLine.getInlines");
+//			} else {
+//				System.out.println("failed...DOMLine.getInlines");
+//			}
+//			DOMFunction test_func = testDOMImage.getFunction(inline_name);
+//			if (test_func.getName() == test_inline) {
+//				System.out.println("passed...DOMFunction.getFunction");
+//			} else {
+//				System.out.println("failed...DOMFunction.getFunction");
+//			}
+//		}
 		
 		if (tag.getType() == tag_type) {
 			System.out.println("\npassed...DOMTag.getType");
